#!/usr/bin/env python
# -*- coding: utf-8 -*-

##  Photini - a simple photo metadata editor.
##  http://github.com/jim-easterbrook/Photini
##  Copyright (C) 2012-17  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import logging
import os
import site
import sys

logger = logging.getLogger(__name__)

# avoid "dll Hell" on Windows by getting PyEnchant to use PyGObject's
# copy of libenchant and associated libraries
if sys.platform == 'win32':
    # disable PyEnchant's forced use of its bundled DLLs
    sys.platform = 'win32x'
    # add gnome DLLs to PATH
    for name in site.getsitepackages():
        gnome_path = os.path.join(name, 'gnome')
        if os.path.isdir(gnome_path):
            os.environ['PATH'] = gnome_path + ';' + os.environ['PATH']
            break

try:
    import enchant
    enchant_version = 'enchant {}'.format(enchant.__version__)
except ImportError:
    enchant = None
    enchant_version = None

if sys.platform == 'win32x':
    # reset sys.platform
    sys.platform = 'win32'
    # using PyGObject's copy of libenchant means it won't find the
    # dictionaries installed with PyEnchant
    if enchant:
        for name in site.getsitepackages():
            dict_path = os.path.join(
                name, 'enchant', 'share', 'enchant', 'myspell')
            if os.path.isdir(dict_path):
                enchant.set_param('enchant.myspell.dictionary.path', dict_path)
                break

from photini.pyqt import Qt, QtCore, QtGui, QtWidgets

class SpellCheck(QtCore.QObject):
    new_dict = QtCore.pyqtSignal()

    def __init__(self, *arg, **kw):
        super(SpellCheck, self).__init__(*arg, **kw)
        self.config_store = QtWidgets.QApplication.instance().config_store
        self.enable(eval(self.config_store.get('spelling', 'enabled', 'True')))
        self.set_dict(self.config_store.get('spelling', 'language'))

    @staticmethod
    def available_languages():
        if not enchant:
            return []
        result = enchant.list_languages()
        result.sort()
        return result

    def current_language(self):
        if self.dict:
            return self.dict.tag
        return ''

    @QtCore.pyqtSlot(bool)
    def enable(self, enabled):
        self.enabled = bool(enchant) and enabled
        self.config_store.set('spelling', 'enabled', str(self.enabled))
        self.new_dict.emit()

    @QtCore.pyqtSlot(QtWidgets.QAction)
    def set_language(self, action):
        self.set_dict(action.text().replace('&', ''))

    def set_dict(self, tag):
        if tag:
            logger.debug('Setting dictionary %s', tag)
        if tag and enchant and enchant.dict_exists(tag):
            self.dict = enchant.Dict(tag)
        else:
            if tag:
                logger.warning('Failed to set dictionary %s', tag)
            self.dict = None
        self.config_store.set('spelling', 'language', self.current_language())
        self.new_dict.emit()



import os
import shutil
import site
import subprocess
import sys

import gi

site_packages = os.path.dirname(os.path.dirname(gi.__file__))

# remove unneeded packages
for line in subprocess.check_output(['pip', 'list', '--format=legacy'],
                                    universal_newlines=True).splitlines():
    package = str(line).split()[0]
    if package in ('appdirs', 'certifi', 'cffi', 'chardet', 'flickrapi', 'idna',
                   'keyring', 'numpy', 'oauthlib', 'opencv-python', 'pgi',
                   'photini', 'Pillow', 'pip', 'pkginfo', 'pycparser', 'pydbus',
                   'pyenchant', 'pygobject','PyQt5', 'pywin32',
                   'pywin32-ctypes', 'requests', 'requests-oauthlib',
                   'requests-toolbelt', 'setuptools', 'six', 'urllib3',
                   'wheel', 'winpython'):
        continue
    subprocess.check_call(['pip', 'uninstall', '-y', package])

# install OpenCV
subprocess.check_call([
    'pip', 'install',
    'C:/Users/Jim/Downloads/opencv_python-3.1.0-cp34-cp34m-win32.whl'])

# remove surplus big files
for name in ('opencv_ffmpeg310.dll', 'PyQt5/qml'):
    path = os.path.join(site_packages, name)
    if not os.path.exists(path):
        continue
    print('delete', path)
    if os.path.isdir(path):
        shutil.rmtree(path)
    else:
        os.unlink(path)

# copy newer exiv2 dlls
gnome = os.path.join(site_packages, 'gnome')
for name in ('libgexiv2-2', 'libexiv2',
             'libgcc_s_dw2-1', 'libstdc++-6',
             'libexpat-1', 'libiconv-2', 'zlib1'):
    dll = name + '.dll'
    print('copying', dll)
    shutil.copy2(os.path.join('C:/msys32/mingw32/bin', dll), gnome)



# -*- coding: utf-8 -*-
#  Photini - a simple photo metadata editor.
#  http://github.com/jim-easterbrook/Photini
#  Copyright (C) 2012-16  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
#  This program is free software: you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see
#  <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import logging
import logging.handlers
import os
import sys

from photini.pyqt import qt_version_info, QtCore, QtWidgets

class OutputInterceptor(object):
    def __init__(self, name, stream):
        self.logger = logging.getLogger(name)
        self.stream = stream
        self.flush = self.stream.flush
        self.fileno = self.stream.fileno

    def write(self, msg):
        self.stream.write(msg)
        msg = msg.strip()
        if 'WARNING' in msg:
            self.logger.warning(msg)
        elif msg:
            self.logger.info(msg)


class StreamProxy(QtCore.QObject):
    # only the GUI thread is allowed to write messages in the
    # LoggerWindow, so this class acts as a proxy, passing messages
    # over Qt signal/slot for thread safety
    flush_text = QtCore.pyqtSignal()
    write_text = QtCore.pyqtSignal(str)

    def write(self, msg):
        msg = msg.strip()
        if msg:
            self.write_text.emit(msg)

    def flush(self):
        self.flush_text.emit()


class LoggerFilter(object):
    def __init__(self, verbosity):
        self.threshold = min(logging.ERROR, (6 - verbosity) * 10)

    def filter(self, record):
        # reduce severity of non-Photini messages
        if (record.levelno < self.threshold and
                not record.name.startswith('photini')):
            return 0
        return 1


class LoggerWindow(QtWidgets.QWidget):
    def __init__(self, verbose, *arg, **kw):
        super(LoggerWindow, self).__init__(*arg, **kw)
        QtWidgets.QApplication.instance().aboutToQuit.connect(self.shutdown)
        self.setWindowTitle(self.tr("Photini error logging"))
        self.setLayout(QtWidgets.QGridLayout())
        self.layout().setRowStretch(0, 1)
        self.layout().setColumnStretch(0, 1)
        # main dialog area
        self.text = QtWidgets.QTextEdit()
        self.text.setReadOnly(True)
        self.text.setMinimumWidth(500)
        self.layout().addWidget(self.text, 0, 0, 1, 3)
        # save button
        save_button = QtWidgets.QPushButton(self.tr('Save'))
        save_button.clicked.connect(self.save)
        self.layout().addWidget(save_button, 1, 1)
        # dismiss button
        dismiss_button = QtWidgets.QPushButton(self.tr('Dismiss'))
        dismiss_button.clicked.connect(self.hide)
        self.layout().addWidget(dismiss_button, 1, 2)
        # Python logger
        self.logger = logging.getLogger('')
        for handler in list(self.logger.handlers):
            self.logger.removeHandler(handler)
        self.logger.setLevel(max(logging.ERROR - (verbose * 10), 1))
        self.stream_proxy = StreamProxy(self)
        self.stream_proxy.write_text.connect(self.write)
        self.stream_proxy.flush_text.connect(self.flush)
        handler = logging.StreamHandler(self.stream_proxy)
        handler.setFormatter(logging.Formatter(
            '%(asctime)s: %(levelname)s: %(name)s: %(message)s',
            datefmt='%H:%M:%S'))
        if verbose > 0:
            handler.addFilter(LoggerFilter(verbose))
        self.logger.addHandler(handler)
        # intercept stdout and stderr, if they exist
        if sys.stderr:
            sys.stderr = OutputInterceptor('stderr', sys.stderr)
        if sys.stdout:
            sys.stdout = OutputInterceptor('stdout', sys.stdout)

    @QtCore.pyqtSlot()
    def shutdown(self):
        self.stream_proxy.write_text.disconnect()
        self.stream_proxy.flush_text.disconnect()
        for handler in list(self.logger.handlers):
            self.logger.removeHandler(handler)

    @QtCore.pyqtSlot()
    def save(self):
        file_name = QtWidgets.QFileDialog.getSaveFileName(
            self, self.tr('Save log file'),
            os.path.expanduser('~/photini_log.txt'))
        if qt_version_info >= (5, 0):
            file_name = file_name[0]
        with open(file_name, 'w') as of:
            of.write(self.text.toPlainText())

    @QtCore.pyqtSlot(str)
    def write(self, msg):
        self.text.append(msg)

    @QtCore.pyqtSlot()
    def flush(self):
        if self.isHidden():
            self.show()
        self.raise_()
